import Plotly from 'plotly.js-dist';
import { DCCRealListQuantity } from '../dccQuantity.js';

const palette = [
  '#1f77b4',
  '#ff7f0e',
  '#2ca02c',
  '#d62728',
  '#9467bd',
  '#8c564b',
  '#e377c2',
  '#7f7f7f',
  '#bcbd22',
  '#17becf'
];

const lightPalette = [
  '#c6e2ff',
  '#ffddbf',
  '#c9e6c8',
  '#f4c2c2',
  '#dcd0ff',
  '#e0cda9',
  '#ffccff',
  '#d3d3d3',
  '#e6e6a9',
  '#b3ffff'
];

export function renderMeasurementResults(measurementResults, language) {
  console.debug('renderMeasurementResults called with:', measurementResults);
  const container = document.getElementById('measurementResults');
  container.innerHTML = '';

  if (!measurementResults) {
    console.error('No measurementResults provided.');
    return;
  }

  let results = measurementResults['dcc:measurementResult'];
  console.debug('Raw dcc:measurementResult:', results);
  if (!results) {
    console.error("Missing 'dcc:measurementResult' in measurementResults");
    return;
  }
  if (!Array.isArray(results)) {
    results = [results];
  }
  console.debug('Processed measurement result array:', results);

  const measurementResult = results[0];
  console.debug('Using measurementResult:', measurementResult);

  let resultObj = measurementResult['dcc:results'] && measurementResult['dcc:results']['dcc:result'];
  if (!resultObj) {
    console.error("Missing 'dcc:results' or 'dcc:result' in measurementResult:", measurementResult);
    return;
  }
  if (Array.isArray(resultObj)) {
    resultObj = resultObj[0];
  }
  console.debug('Using result object:', resultObj);

  let resultName = 'Measurement Result';
  if (measurementResult['dcc:name'] && measurementResult['dcc:name']['dcc:content']) {
    let content = measurementResult['dcc:name']['dcc:content'];
    if (Array.isArray(content)) {
      const match = content.find(item => item.$ && item.$.lang === language) || content[0];
      resultName = match._ || match;
    } else {
      resultName = content._ || content;
    }
  }
  console.debug('Result name:', resultName);
  const tabTitle = document.createElement('h2');
  tabTitle.textContent = resultName;
  container.appendChild(tabTitle);

  if (!resultObj['dcc:data'] || !resultObj['dcc:data']['dcc:list']) {
    console.error("Missing 'dcc:data' or 'dcc:list' in result object:", resultObj);
    return;
  }
  const listData = resultObj['dcc:data']['dcc:list'];
  console.debug('List data:', listData);

  let quantityJSONs = [];
  if (listData['dcc:quantity']) {
    quantityJSONs = Array.isArray(listData['dcc:quantity'])
        ? listData['dcc:quantity']
        : [listData['dcc:quantity']];
  }
  if (listData['dcc:measurementMetaData'] && listData['dcc:measurementMetaData']['dcc:metaData']) {
    let metaData = listData['dcc:measurementMetaData']['dcc:metaData'];
    if (!Array.isArray(metaData)) metaData = [metaData];
    metaData.forEach(md => {
      if (md['dcc:data'] && md['dcc:data']['dcc:quantity']) {
        let qs = md['dcc:data']['dcc:quantity'];
        if (!Array.isArray(qs)) qs = [qs];
        quantityJSONs = quantityJSONs.concat(qs);
      }
    });
  }
  console.debug('Combined quantity JSONs:', quantityJSONs);

  const indexQuantities = [];
  const dataQuantities = [];
  const extraInfo = [];
  quantityJSONs.forEach(q => {
    if (q.$ && q.$.refType && q.$.refType.match(/basic_tableIndex/)) {
      indexQuantities.push(new DCCRealListQuantity(q));
    } else {
      dataQuantities.push(new DCCRealListQuantity(q));
      let uncertainty = (new DCCRealListQuantity(q)).getUncertainty();
      let comment = '';
      let conformity = null;
      if (q['dcc:measurementMetaData'] && q['dcc:measurementMetaData']['dcc:metaData']) {
        let md = q['dcc:measurementMetaData']['dcc:metaData'];
        if (!Array.isArray(md)) { md = [md]; }
        md.forEach(item => {
          if (item.$ && item.$.refType && item.$.refType.includes('basic_tableRowComment')) {
            if (item['dcc:description'] && item['dcc:description']['dcc:content']) {
              let desc = item['dcc:description']['dcc:content'];
              if (Array.isArray(desc)) {
                const match = desc.find(d => d.$ && d.$.lang === language) || desc[0];
                comment = match._ || match;
              } else {
                comment = desc._ || desc;
              }
            }
          }
          if (item.$ && item.$.refType && item.$.refType.includes('basic_conformity')) {
            if (item['dcc:conformityXMLList']) {
              conformity = item['dcc:conformityXMLList'].trim().split(/\s+/);
            }
          }
        });
      }
      extraInfo.push({ uncertainty, comment, conformity });
    }
  });
  console.debug('Index Quantities (objects):', indexQuantities);
  console.debug('Data Quantities (objects):', dataQuantities);
  console.debug('Extra info for data quantities:', extraInfo);

  const xAxisContainer = document.createElement('div');
  xAxisContainer.innerHTML = '<strong>Select X-Axis:</strong> ';
  indexQuantities.forEach((q, idx) => {
    let nameStr = q.getName(language) || ('Index ' + idx);
    const radio = document.createElement('input');
    radio.type = 'radio';
    radio.name = 'xAxisSelect';
    radio.value = idx;
    if (idx === 0) radio.checked = true;
    const label = document.createElement('label');
    label.textContent = nameStr;
    label.style.marginRight = '10px';
    xAxisContainer.appendChild(radio);
    xAxisContainer.appendChild(label);
  });
  container.appendChild(xAxisContainer);

  const toleranceToggle = document.createElement('input');
  toleranceToggle.type = 'checkbox';
  toleranceToggle.id = 'toleranceToggle';
  const tolLabel = document.createElement('label');
  tolLabel.htmlFor = 'toleranceToggle';
  tolLabel.textContent = ' Enable tolerance markings';
  const tolContainer = document.createElement('div');
  tolContainer.appendChild(toleranceToggle);
  tolContainer.appendChild(tolLabel);
  container.appendChild(tolContainer);

  const subplotsContainer = document.createElement('div');
  subplotsContainer.id = 'subplotsContainer';
  container.appendChild(subplotsContainer);
  const tableContainer = document.createElement('div');
  tableContainer.id = 'tableContainer';
  container.appendChild(tableContainer);

  function updateVisualization() {
    const selectedRadio = document.querySelector('input[name="xAxisSelect"]:checked');
    if (!selectedRadio) {
      console.error('No X-Axis selection found.');
      return;
    }
    const selectedIndex = selectedRadio.value;
    const xQuantity = indexQuantities[selectedIndex];
    const xValues = xQuantity.getValues();
    const xUnit = xQuantity.getUnit();
    console.debug('Selected X-Axis values:', xValues);
    console.debug('X-Axis unit:', xUnit);

    const headers = [];
    const xHeader = 'X-Axis (selected) (' + xUnit + ')';
    headers.push(xHeader);

    const dataValues = [];
    const commentsArray = [];
    const conformityArray = [];
    const uncertaintiesArray = [];
    dataQuantities.forEach((q, idx) => {
      let header = q.getName(language);
      let unit = q.getUnit();
      if (!header.toLowerCase().includes(" in ")) {
        header = header + " in " + unit;
      }
      headers.push(header);
      headers.push('Comments');
      headers.push(extraInfo[idx] && extraInfo[idx].conformity ? 'Conformity' : '');
      let values = q.getValues();
      if (values.length === 1 && xValues.length > 1) {
        values = new Array(xValues.length).fill(values[0]);
      }
      dataValues.push(values);
      uncertaintiesArray.push(extraInfo[idx] ? extraInfo[idx].uncertainty : null);
      commentsArray.push(extraInfo[idx] ? extraInfo[idx].comment : '');
      conformityArray.push(extraInfo[idx] ? extraInfo[idx].conformity : []);
    });
    console.debug('Table data arrays:', { dataValues, commentsArray, conformityArray, uncertaintiesArray });
    const tableData = [headers];
    for (let i = 0; i < xValues.length; i++) {
      const row = [];
      row.push(xValues[i]);
      dataValues.forEach((values, idx) => {
        let cellValue = values[i] !== undefined ? values[i] : '';
        if (uncertaintiesArray[idx] && uncertaintiesArray[idx][i] !== undefined) {
          cellValue = cellValue + ' ± ' + uncertaintiesArray[idx][i];
        }
        row.push(cellValue);
        row.push(commentsArray[idx] || '');
        row.push(conformityArray[idx] && conformityArray[idx][i] ? conformityArray[idx][i] : '');
      });
      tableData.push(row);
    }
    console.debug('Final table data:', tableData);
    renderTable(tableData);

    const unitGroups = {};
    dataQuantities.forEach((q, idx) => {
      const unit = q.getUnit();
      if (!unitGroups[unit]) {
        unitGroups[unit] = [];
      }
      let header = headers[idx * 3 + 1];
      let values = q.getValues();
      if (values.length === 1 && xValues.length > 1) {
        values = new Array(xValues.length).fill(values[0]);
      }
      const uncertainty = uncertaintiesArray[idx];
      const conformity = conformityArray[idx];
      const traceColor = palette[idx % palette.length];
      unitGroups[unit].push({ name: header, y: values, uncertainty, conformity, color: traceColor, index: idx });
    });
    console.debug('Unit groups for plots:', unitGroups);

    subplotsContainer.innerHTML = '';
    const unitKeys = Object.keys(unitGroups);
    unitKeys.forEach((unit, groupIdx) => {
      const group = unitGroups[unit];
      const graphDiv = document.createElement('div');
      graphDiv.style.width = '100%';
      graphDiv.style.height = '300px';
      subplotsContainer.appendChild(graphDiv);

      const groupTraces = group.map(trace => {
        const hovertemplate = 'X: %{x} ' + xUnit + ' | ' + trace.name + ': %{y} ± %{customdata_unc} ' + unit + ' | Conformity: %{customdata}<extra></extra>';
        return {
          x: xValues,
          y: trace.y,
          error_y: { type: 'data', array: trace.uncertainty || new Array(xValues.length).fill(0), visible: true, color: trace.color },
          type: 'scatter',
          mode: 'lines+markers',
          name: trace.name,
          marker: { color: trace.color },
          hovertemplate: hovertemplate,
          customdata: (trace.conformity && trace.conformity.length === xValues.length) ? trace.conformity : new Array(xValues.length).fill(''),
          customdata_unc: (trace.uncertainty && trace.uncertainty.length === xValues.length) ? trace.uncertainty : new Array(xValues.length).fill('0')
        };
      });

      let xaxisTitle = '';
      if (groupIdx === unitKeys.length - 1) {
        let xLabel = 'X: ' + xQuantity.getName(language);
        xaxisTitle = xLabel + ' in ' + xUnit;
      }
      const layout = {
        xaxis: { title: { text: xaxisTitle, font: { size: 18, family: 'Arial', color: 'black' } }, tickfont: { family: 'Arial', size: 14, color: 'black' } },
        yaxis: { title: { text: unit, font: { size: 18, family: 'Arial', color: 'black' } }, tickfont: { family: 'Arial', size: 14, color: 'black' } },
        hovermode: 'closest',
        margin: { t: 20, b: 40 }
      };

      Plotly.newPlot(graphDiv, groupTraces, layout).then(() => {
        console.debug('Plot rendered for unit:', unit);
        const caption = document.createElement('div');
        caption.innerHTML = '<b>' + group[0].name +'</b>';
        caption.style.textAlign = 'center';
        caption.style.marginBottom = '5px';
        graphDiv.parentNode.insertBefore(caption, graphDiv);
      });

      graphDiv.on('plotly_hover', function(data) {
        if (data.points && data.points.length > 0) {
          const pointIndex = data.points[0].pointIndex + 1;
          highlightTableRow(pointIndex);
        }
      });
      graphDiv.on('plotly_unhover', function() {
        clearTableRowHighlights();
      });
    });
  }

  function renderTable(tableData) {
    const tableContainer = document.getElementById('tableContainer');
    tableContainer.innerHTML = '';
    const table = document.createElement('table');
    tableData.forEach((rowData, rowIndex) => {
      const tr = document.createElement('tr');
      rowData.forEach((cellData, cellIndex) => {
        const cell = document.createElement(rowIndex === 0 ? 'th' : 'td');
        // For headers, allow HTML rendering (dsiUnits output) by setting innerHTML.
        if (rowIndex === 0) {
          cell.innerHTML = cellData;
        } else {
          cell.textContent = cellData;
        }
        cell.style.padding = '4px';
        cell.style.border = '1px solid #ccc';
        if (rowIndex === 0 && cellIndex > 0) {
          const dataIndex = Math.floor((cellIndex - 1) / 3);
          cell.style.backgroundColor = lightPalette[dataIndex % lightPalette.length];
        }
        tr.appendChild(cell);
      });
      tr.addEventListener('mouseover', () => { tr.style.backgroundColor = '#eef'; });
      tr.addEventListener('mouseout', () => { tr.style.backgroundColor = ''; });
      table.appendChild(tr);
    });
    tableContainer.appendChild(table);
  }

  function highlightTableRow(rowIndex) {
    const rows = document.getElementById('tableContainer').querySelectorAll('tr');
    if (rows[rowIndex]) {
      rows[rowIndex].style.backgroundColor = '#fee';
    }
  }

  function clearTableRowHighlights() {
    const rows = document.getElementById('tableContainer').querySelectorAll('tr');
    rows.forEach(row => row.style.backgroundColor = '');
  }

  updateVisualization();
  const radios = document.querySelectorAll('input[name="xAxisSelect"]');
  radios.forEach(radio => {
    radio.addEventListener('change', updateVisualization);
  });
  toleranceToggle.addEventListener('change', () => {
    console.log('Tolerance toggle:', toleranceToggle.checked);
  });
}
